﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using Buddy.Common;
using Buddy.Coroutines;
using Buddy.Wildstar.BotCommon;
using Buddy.Wildstar.Engine;
using Buddy.Wildstar.Game;
using log4net;

namespace ExpBuff
{
    public class WildstarPlugin : CoroutinePulsable, IPlugin
    {
        private static ILog Log = LogManager.GetLogger("ExpBuff");
        public string Name { get { return "ExpBuff"; } }
        public string Author { get { return "King Dwink"; } }
        public Version Version { get { return new Version(0, 0, 1); } }
        public bool Enabled { get; set; }

        public override async Task CoroutineImplementation()
        {
            #region ExpBuff
            // If we have the buff, we don't need to do anything.
            if (GameManager.LocalPlayer.Buffs.Any(b => b.Name == "Flask of Experience"))
                return;

            // Make sure we have the flask.
            InventoryItem item = GameManager.Inventory.Bags.Items.FirstOrDefault(i => i.Id == 31443);
            if (item == null || !item.IsValid)
                return;

            // Pause the bot pulse and stop our character.
            GameEngine.BotPulsator.Suspend(GameEngine.CurrentBot, TimeSpan.FromMilliseconds(2000)); // Consider shortening this? Seems pretty long.
            Buddy.Wildstar.BotCommon.Navigator.Stop(false); // No need to clear path, keep set to false.
            await Coroutine.Wait(1000, () => !GameManager.LocalPlayer.IsMoving); // Wait for character to stop.

            // Use the flask.
            item.Use();
            await Coroutine.Wait(1000, () => GameManager.LocalPlayer.IsCasting); // Wait for casting to start.
            await Coroutine.Wait(5000, () => !GameManager.LocalPlayer.IsCasting); // Wait for casting to stop.
            #endregion
        }


        public override void OnRegistered() { }
        public override void OnUnregistered() { }

        public void Initialize()
        {
            GameEngine.BotPulsator.RegisterForPulse(this);
        }


        public override bool CanBePulsed
        {
            get
            {
                var Me = GameManager.LocalPlayer;
                // Loading screen.
                if (GameManager.WorldLoadState != WorldLoadState.InWorld)
                    return false;

                // Sanity checks.
                if (Me == null || !Me.IsValid)
                    return false;

                if (Me.IsDead || !Me.IsAlive)
                    return false;

                if (Me.IsInCombat)
                    return false;

                return true;
            }
        }

        private void ToggleMe()
        {
            Enabled = !Enabled;
        }

        public void Uninitialize()
        {
            GameEngine.BotPulsator.UnregisterForPulse(this);
        }
    }
}
